/*************************************************************************
*  File: vehicles.c                                    Part of CircleMUD *
*  Usage: Vechicle related code						 *
*									 *
*  All rights reserved.  See license.doc for complete information.	 *
*									 *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University*
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.              *
*  Vehicle.c written by Chris Jacobson <fear@athenet.net>		 *
*************************************************************************/

#include "conf.h"
#include "sysdep.h"


#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "screen.h"
#include "house.h"
#include "constants.h"

extern struct room_data *world;
extern struct char_data *character_list;
extern struct descriptor_data *descriptor_list;
extern struct index_data *obj_index;
extern int rev_dir[];
extern const char *room_bits[];
extern const char *region_names[];

struct obj_data *get_obj_in_list_type(int type, struct obj_data *list);
ACMD(do_look);
void list_obj_to_char(struct obj_data * list, struct char_data * ch, int mode, bool show);
void list_char_to_char(struct char_data * list, struct char_data * ch);
extern struct obj_data *get_obj_in_list_type(int type, struct obj_data *list);
struct obj_data *find_vehicle(int roomNum);

typedef struct {
	int dirNum;
	char* dirCmd;
} dirParseStruct;

#define DRIVE_DIRS 6
#define PILOT_DIRS 6

#ifndef EXITN
#  define EXITN(room, door)		(world[room].dir_option[door])
#endif

struct obj_data *find_vehicle_by_vnum(int vnum) 
{
  extern struct obj_data * object_list;
  struct obj_data * i;

  for (i = object_list; i; i = i->next)
    if (GET_OBJ_TYPE(i) == ITEM_VEHICLE)
      if (GET_OBJ_VNUM(i) == vnum)
        return i;
    
  return 0;
}

/* Search the given list for an object type, and return a ptr to that obj*/
struct obj_data *get_obj_in_list_type(int type, struct obj_data *list) 
{
  struct obj_data * i;

  for (i = list; i; i = i->next_content)
    if (GET_OBJ_TYPE(i) == type)
      return i;

  return NULL;
}

void auto_exits_by_rnum(struct char_data * ch, int is_in)
{
  int door, slen = 0;
  char buf[MAX_INPUT_LENGTH];

  *buf = '\0';

  send_to_char(ch, "%s[ Exits: ", CCCYN(ch, C_NRM));

  for (door = 0; door < NUM_OF_DIRS; door++)

  if (EXITN(is_in, door) && EXITN(is_in, door)->to_room != NOWHERE) {
    if (EXIT_FLAGGED(EXITN(is_in, door), EX_CLOSED))
      send_to_char(ch, "&r%c&c ", LOWER(*dirs[door]));
    else
      send_to_char(ch, "&c%c ", LOWER(*dirs[door]));
  }

  slen++;

  send_to_char(ch, "%s]%s\r\n", slen ? "" : "None!", CCNRM(ch, C_NRM));  
}


void view_room_by_rnum(struct char_data * ch, int is_in) 
{
  char buf[MAX_INPUT_LENGTH];

  if (IS_DARK(is_in) && !CAN_SEE_IN_DARK(ch)) {
    send_to_char(ch, "It is pitch black...\r\n");
    return;
  } else if (AFF_FLAGGED(ch, AFF_BLIND)) {
    send_to_char(ch, "You see nothing but infinite darkness...\r\n");
    return;
  }
  send_to_char(ch, "%s", CCCYN(ch, C_NRM));
  if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_ROOMFLAGS)) {
    sprintbit((long) ROOM_FLAGS(is_in), room_bits, buf, sizeof(buf));
    send_to_char(ch,"[%5d] %s [ %s] ", world[is_in].number,
     world[is_in].name, buf);
  } else
    send_to_char(ch, "%s", world[is_in].name);
		
  send_to_char(ch, "%s\r\n", CCNRM(ch, C_NRM));

  if ((!IS_NPC(ch) && !PRF_FLAGGED(ch, PRF_BRIEF)) || 
   ROOM_FLAGGED(is_in, ROOM_DEATH))
    send_to_char(ch, "%s", world[is_in].description);

  /* autoexits */
  if (!IS_NPC(ch) && PRF_FLAGGED(ch, PRF_AUTOEXIT))
    auto_exits_by_rnum(ch, is_in);

  /* now list characters & objects */
  send_to_char(ch, "%s", CCGRN(ch, C_NRM));
  list_obj_to_char(world[is_in].contents, ch, 0, FALSE);
  send_to_char(ch, "%s", CCYEL(ch, C_NRM));
  list_char_to_char(world[is_in].people, ch);
  send_to_char(ch, "%s", CCNRM(ch, C_NRM));

}


ACMD(do_drive) 
{
  int x, dir;
  const dirParseStruct dirParse[6] = {
   {SCMD_NORTH -1, "north"},
   {SCMD_SOUTH -1, "south"},
   {SCMD_EAST -1,  "east"},
   {SCMD_WEST -1,  "west"},
   {SCMD_UP -1,    "up"},
   {SCMD_DOWN -1,  "down"}
  };
  struct obj_data *obj, *vehicle, *temp, *controls, *vehicle_in_out;
  char arg[MAX_INPUT_LENGTH];
  char buf[MAX_INPUT_LENGTH];
  char buf2[MAX_INPUT_LENGTH];
  int found = FALSE, j;

  controls = get_obj_in_list_type(ITEM_CONTROL, world[ch->in_room].contents);
  if (!controls) {
    for (obj = ch->carrying; obj && !found; obj = obj->next_content) {
      if (CAN_SEE_OBJ(ch, obj) && GET_OBJ_TYPE(obj) == ITEM_CONTROL) {
        controls = obj;
        found = TRUE;
      }
    }
    if (!controls) {
      for (j = 0; j < NUM_WEARS && !found; j++)
        if (GET_EQ(ch, j) && CAN_SEE_OBJ(ch, GET_EQ(ch, j)) && 
         GET_OBJ_TYPE(GET_EQ(ch, j)) == ITEM_CONTROL) {
          controls = GET_EQ(ch, j);
          found = TRUE;
        }
      if (!controls) {
        send_to_char(ch,"ERROR! Vehicle controls present yet not present!\r\n");
        return;
      }
    }
  }

  vehicle = find_vehicle_by_vnum(GET_OBJ_VAL(controls, 0));
  if (!vehicle) {
    send_to_char(ch, "ERROR!  Vehicle has been lost somehow!\r\n");
    return;
  }

  temp=vehicle;
	
  if (IS_AFFECTED(ch, AFF_BLIND)) { /* Blind characters can't drive! */
    send_to_char(ch, "You can't see the controls!\r\n");
    return;
  }

  two_arguments(argument, arg, buf2); /* Gotta give us a direction... */

  if (!*arg) {
    send_to_char(ch, "Drive which direction?\r\n");
    return;
  }
	
  /* Driving Into another Vehicle */
  if (is_abbrev(arg, "into")) {
    int was_in, is_in, is_going_to;
		
    if (!*buf2) {
      send_to_char(ch, "Drive into what?\r\n");
      return;
    }
		
    vehicle_in_out = get_obj_in_list_vis(ch, buf2, NULL, world[vehicle->in_room].contents);

    if (!vehicle_in_out) {
      send_to_char(ch, "Nothing here by that name!\r\n");
      return;
    }
		
    if (GET_OBJ_TYPE(vehicle_in_out) != ITEM_VEHICLE) {
      send_to_char(ch, "Thats not a vehicle.\r\n");
      return;
    }
		
    is_going_to = real_room(GET_OBJ_VAL(vehicle_in_out, 0));
		
    if (!IS_SET_AR(ROOM_FLAGS(is_going_to), ROOM_VEHICLE)) {
      send_to_char(ch, "That vehicle can't carry other vehicles.");
      return;
    }

    sprintf(buf, "%s enters %s.\n\r", vehicle->short_description, 
     vehicle_in_out->short_description);
    send_to_room(vehicle->in_room, buf);

    was_in = vehicle->in_room;
    obj_from_room(vehicle);
    obj_to_room(vehicle, is_going_to);
				
    is_in = vehicle->in_room;
		
    if (ch->desc != NULL)
      view_room_by_rnum(ch, is_in);
					
    sprintf(buf, "%s enters.\r\n", vehicle->short_description);
    send_to_room(is_in, buf);
    return;
    } else if (is_abbrev(arg, "out")) {
      struct obj_data *hatch;
		
      hatch = get_obj_in_list_type(ITEM_HATCH,world[vehicle->in_room].contents);
        if (!hatch) {
          send_to_char(ch, "Nowhere to drive out of.\r\n");
          return;
        }
		
        vehicle_in_out = find_vehicle_by_vnum(GET_OBJ_VAL(hatch, 0));
        if (!vehicle_in_out) {
          send_to_char(ch, "ERROR! Vehicle to drive out of doesn't exist!\r\n");
          return;
        }
		
        sprintf(buf, "%s exits %s.\r\n", vehicle->short_description, 
         vehicle_in_out->short_description);
        send_to_room(vehicle->in_room, buf);
		
        obj_from_room(vehicle);
        obj_to_room(vehicle, vehicle_in_out->in_room);
		
        if (ch->desc != NULL)
          view_room_by_rnum(ch, vehicle->in_room);
		
        sprintf(buf, "%s drives out of %s.\r\n", vehicle->short_description,
         vehicle_in_out->short_description);
        send_to_room(vehicle->in_room, buf);
        return;
    } else for(x=0; x < (GET_OBJ_VAL(vehicle,1) ? PILOT_DIRS : DRIVE_DIRS); x++)
    /* Drive in a direction... */
    if (is_abbrev(arg, dirParse[x].dirCmd)) {
      dir = dirParse[x].dirNum;
      /* Ok we found the direction! */
      if (ch == NULL || dir < 0 || dir >= NUM_OF_DIRS)
        /* But something is invalid */
        return;

      else if (!EXIT(vehicle, dir) || EXIT(vehicle, dir)->to_room == NOWHERE) {
        /* But there is no exit that way */
        send_to_char(ch, "Alas, you cannot go that way...\r\n");
        return;
      } else if (IS_SET(EXIT(vehicle, dir)->exit_info, EX_CLOSED)) {
        /* But the door is closed */
        if (EXIT(vehicle, dir)->keyword)
          send_to_char(ch, "The %s seems to be closed.\r\n", fname(EXIT(vehicle,           dir)->keyword));
        else
          send_to_char(ch, "It seems to be closed.\r\n");

        return;
      } else if (!IS_SET_AR(ROOM_FLAGS(EXIT(vehicle,dir)->to_room),ROOM_VEHICLE)) {
        /* But the vehicle can't go that way*/
        send_to_char(ch, "The vehicle can't manage that terrain.\r\n");
        return;
        } else {
          /* But nothing!  Let's go that way! */
          int was_in, is_in;

          sprintf(buf, "%s leaves %s.\n\r", temp->short_description, dirs[dir]);
          send_to_room(vehicle->in_room, buf);

          was_in = vehicle->in_room;
          obj_from_room(vehicle);
          obj_to_room(vehicle, world[was_in].dir_option[dir]->to_room);

          is_in = vehicle->in_room;

          if (ch->desc != NULL)
            view_room_by_rnum(ch, is_in);

          sprintf(buf, "%s enters from the %s.\r\n",
           temp->short_description, dirs[rev_dir[dir]]);
          send_to_room(is_in, buf);
          return;
        }
      }
      send_to_char(ch, "Thats not a valid direction.\r\n");
      return;
}
